@@ -57,7 +57,7 @@
 (export gnc:exchange-by-euro)
 (export gnc:exchange-if-same)
 (export gnc:make-exchange-function)
-(export gnc:exchange-by-pricevalue-helper)
+(export gnc:exchange-by-pricevalue-helper) ;deprecated
 (export gnc:exchange-by-pricedb-helper)
 (export gnc:exchange-by-pricedb-latest )
 (export gnc:exchange-by-pricedb-nearest)
@@ -66,7 +66,7 @@
 (export gnc:case-exchange-time-fn)
 (export gnc:sum-collector-commodity)
 (export gnc:sum-collector-stocks)
-(export gnc-commodity-collector-contains-commodity?)
+(export gnc-commodity-collector-contains-commodity?) ;deprecated
 
 ;; options-utilities.scm
 
@@ -75,10 +75,10 @@
 (export gnc:options-add-interval-choice!)
 (export gnc:options-add-account-levels!)
 (export gnc:options-add-account-selection!)
-(export gnc:options-add-include-subaccounts!)
-(export gnc:options-add-group-accounts!)
+(export gnc:options-add-include-subaccounts!) ;deprecated
+(export gnc:options-add-group-accounts!)      ;deprecated
 (export gnc:options-add-currency!)
-(export gnc:options-add-currency-selection!)
+(export gnc:options-add-currency-selection!)  ;deprecated
 (export gnc:options-add-price-source!)
 (export gnc:options-add-plot-size!)
 (export gnc:options-add-marker-choice!)
@@ -686,15 +686,16 @@
 (export gnc:accounts-get-commodities)
 (export gnc:get-current-account-tree-depth)
 (export gnc:acccounts-get-all-subaccounts)
-(export gnc:make-stats-collector)
-(export gnc:make-drcr-collector)
+(export gnc:make-stats-collector)       ;deprecated
+(export gnc:make-drcr-collector)        ;deprecated
 (export gnc:make-value-collector)
-(export gnc:make-number-collector)
+(export gnc:make-number-collector)      ;deprecated
 (export gnc:make-commodity-collector)
 (export gnc:commodity-collector-get-negated)
-(export gnc:commodity-collectorlist-get-merged)
+(export gnc:commodity-collectorlist-get-merged) ;deprecated
 (export gnc-commodity-collector-commodity-count)
 (export gnc:account-get-balance-at-date)
+(export gnc:account-get-balances-at-dates)
 (export gnc:account-get-comm-balance-at-date)
 (export gnc:account-get-comm-value-interval)
 (export gnc:account-get-comm-value-at-date)
@@ -718,12 +719,14 @@
 (export gnc:report-finished)
 (export gnc:accounts-count-splits)
 (export gnc-commodity-collector-allzero?)
+(export gnc:monetary+)
+(export gnc:monetaries-add)
 (export gnc:account-get-trans-type-balance-interval)
 (export gnc:account-get-trans-type-balance-interval-with-closing)
-(export gnc:account-get-total-flow)
+(export gnc:account-get-total-flow)     ;deprecated
 (export gnc:account-get-pos-trans-total-interval)
 (export gnc:account-get-trans-type-splits-interval)
-(export gnc:double-col)
+(export gnc:double-col)                 ;deprecated
 (export gnc:budget-get-start-date)
 (export gnc:budget-get-end-date)
 (export gnc:budget-account-get-net)
@@ -737,6 +740,8 @@
 (export gnc:select-assoc-account-balance)
 (export gnc:get-assoc-account-balances-total)
 (export make-file-url)
+(export gnc:strify)
+(export gnc:pk)
 
 (load-from-path "commodity-utilities")
 (load-from-path "html-barchart")
