@@ -6,10 +6,29 @@
 ;;  these calculations and accepts no responsibility for direct
 ;;  or indirect losses incurred as a result of using this software.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; This program is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU General Public License as
+;; published by the Free Software Foundation; either version 2 of
+;; the License, or (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program; if not, contact:
+;;
+;; Free Software Foundation           Voice:  +1-617-542-5942
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 
 (define-module (gnucash report standard-reports average-balance))
 (use-modules (srfi srfi-1))
-(use-modules (gnucash main))
+(use-modules (gnucash utilities))
 (use-modules (gnucash gnc-module))
 (use-modules (gnucash gettext))
 (gnc:module-load "gnucash/report/report-system" 0)
@@ -23,6 +42,8 @@
 (define optname-price-source (N_ "Price Source"))
 (define optname-subacct (N_ "Include Sub-Accounts"))
 (define optname-internal (N_ "Exclude transactions between selected accounts"))
+(define optname-plot-width (N_ "Plot Width"))
+(define optname-plot-height (N_ "Plot Height"))
 
   ;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Options
@@ -108,8 +129,8 @@
        (vector 'GLPlot (N_ "Gain/Loss") (N_ "Gain And Loss.")))))
 
     (gnc:options-add-plot-size! 
-     options gnc:pagename-display (N_ "Plot Width") (N_ "Plot Height")
-     "d" 400 400)
+     options gnc:pagename-display 
+     optname-plot-width optname-plot-height "d" (cons 'percent 100.0) (cons 'percent 100.0))
 
     ;; Set the general page as default option tab
     (gnc:options-set-default-section options gnc:pagename-general)      
@@ -127,167 +148,6 @@
         (_ "Maximum") (_ "Minimum") (_ "Gain") 
         (_ "Loss") (_ "Profit") ))
 
-;; analyze-splits crunches a split list into a set of period
-;; summaries.  Each summary is a list of (start-date end-date
-;; avg-bal max-bal min-bal total-in total-out net) if multiple
-;; accounts are selected the balance is the sum for all.  Each
-;; balance in a foreign currency will be converted to a double in
-;; the report-currency by means of the monetary->double
-;; function. 
-(define (analyze-splits splits start-bal-double 
-                        start-date end-date interval monetary->double
-			internal)
-  (let ((interval-list 
-         (gnc:make-date-interval-list start-date end-date interval))
-        (data-rows '()))
-    
-    (define (output-row interval-start 
-                        interval-end 
-                        stats-accum 
-                        minmax-accum
-                        gain-loss-accum)
-      (set! data-rows
-            (cons 
-             (list (gnc-print-date interval-start)
-                   (gnc-print-date interval-end)
-                   (/ (stats-accum 'total #f)
-                      (gnc:timepair-delta interval-start 
-                                          interval-end))
-                   (minmax-accum 'getmax #f)
-                   (minmax-accum 'getmin #f)
-                   (gain-loss-accum 'debits #f) 
-                   (gain-loss-accum 'credits #f)
-                   (- (gain-loss-accum 'debits #f)
-                      (gain-loss-accum 'credits #f)))
-             data-rows)))
-    
-    ;; Returns a double which is the split value, correctly
-    ;; exchanged to the current report-currency. We use the exchange
-    ;; rate at the 'date'.
-    (define (get-split-value split date)
-      (monetary->double
-       (gnc:make-gnc-monetary
-        (xaccAccountGetCommodity (xaccSplitGetAccount split))
-        (xaccSplitGetAmount split))
-       date))
-    
-    ;; calculate the statistics for one interval - returns a list 
-    ;;  containing the following: 
-    ;; min-max acculumator
-    ;; average-accumulator
-    ;; gain-loss accumulator
-    ;; final balance for this interval
-    ;; splits remaining to be processed.
-    
-    ;; note that it is assumed that every split in in the list
-    ;; has a date >= from 
-
-    (define (process-interval splits from to start-balance)
-
-      (let ((minmax-accum (gnc:make-stats-collector))
-            (stats-accum (gnc:make-stats-collector))
-            (gain-loss-accum (gnc:make-drcr-collector))
-            (last-balance start-balance)
-            (last-balance-time from))
-        
-        
-        (define (update-stats  split-amt split-time)
-          (let ((time-difference (gnc:timepair-delta 
-                                  last-balance-time
-                                  split-time)))
-            (stats-accum 'add (* last-balance time-difference))
-            (set! last-balance (+ last-balance split-amt))
-            (set! last-balance-time split-time)
-            (minmax-accum 'add last-balance)
-            (gain-loss-accum 'add split-amt)))
-
-        (define (split-recurse)
-          (if (or (null? splits) (gnc:timepair-gt 
-                                  (gnc-transaction-get-date-posted
-                                   (xaccSplitGetParent
-                                    (car splits))) to)) 
-              #f
-              (let* 
-                  ((split (car splits))
-                   (split-time (gnc-transaction-get-date-posted
-                                (xaccSplitGetParent split)))
-                   ;; FIXME: Which date should we use here? The 'to'
-                   ;; date? the 'split-time'?
-		   (split-amt (get-split-value split split-time))
-		   (next (cdr splits)))
-		
-		(if 
-		 ;; Check whether this split and next one are a pair
-		 ;; from the same transaction, and the only ones in
-		 ;; this transaction.
-		 ;; If they are and the flag is set appropriately,
-		 ;; then skip both.
-		 (or internal
-		     (null? next)
-		     (let* ((next-split (car next))
-			    (trans (xaccSplitGetParent split))
-			    (next-trans (xaccSplitGetParent next-split))
-			    (count (xaccTransCountSplits trans)))
-		       (not (and (eqv? count 2)
-				 (equal? trans next-trans)))))                
-		 (begin
-		  (gnc:debug "split " split)
-		  (gnc:debug "split-time " split-time)
-		  (gnc:debug "split-amt " split-amt)
-		  ;; gnc:debug converts its input to a string before
-		  ;; deciding whether to print it, and converting
-		  ;; |splits| to a string is O(N) in its length.  Since
-		  ;; this code runs for every split, leaving that
-		  ;; gnc:debug in makes the whole thing O(N^2) in number
-		  ;; of splits.  If someone really needs this output,
-		  ;; they should uncomment the gnc:debug call.
-					; (gnc:debug "splits " splits)
-		  (update-stats split-amt split-time)
-		  (set! splits next)
-		  (split-recurse))
-		 (begin
-		  (set! splits (cdr next))
-		  (split-recurse))))))
-
-                                        ;  the minmax accumulator
-
-        (minmax-accum 'add start-balance)
-
-        (if (not (null? splits))
-            (split-recurse))
-
-        ;; insert a null transaction at the end of the interval
-        (update-stats 0.0 to)
-        (list minmax-accum stats-accum gain-loss-accum last-balance splits)))
-    
-    
-    (for-each
-     (lambda (interval)
-       (let* 
-           
-           ((interval-results 
-             (process-interval 
-              splits 
-              (car interval) 
-              (cadr interval)
-              start-bal-double))
-            (min-max-accum (car interval-results))
-            (stats-accum (cadr interval-results))
-            (gain-loss-accum (caddr interval-results))
-            (last-bal (cadddr interval-results))
-            (rest-splits (list-ref interval-results 4)))
-
-         (set! start-bal-double last-bal)
-         (set! splits rest-splits)
-         (output-row (car interval) 
-                     (cadr interval) 
-                     stats-accum 
-                     min-max-accum gain-loss-accum)))
-     interval-list)
-    
-    
-    (reverse data-rows)))
-
 
   ;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Renderer
@@ -302,10 +162,10 @@
   (gnc:report-starting reportname)
   (let* ((report-title (get-option gnc:pagename-general 
                                    gnc:optname-reportname))
-         (begindate (gnc:timepair-start-day-time
+         (begindate (gnc:time64-start-day-time
                      (gnc:date-option-absolute-time 
                       (get-option gnc:pagename-general optname-from-date))))
-         (enddate (gnc:timepair-end-day-time 
+         (enddate (gnc:time64-end-day-time 
                    (gnc:date-option-absolute-time 
                     (get-option gnc:pagename-general optname-to-date))))
          (stepsize (gnc:deltasym-to-delta (get-option gnc:pagename-general optname-stepsize)))
@@ -317,7 +177,11 @@
          (internal-included (not (get-option gnc:pagename-accounts optname-internal)))
          (accounts   (get-option gnc:pagename-accounts (N_ "Accounts")))
          (dosubs?    (get-option gnc:pagename-accounts optname-subacct))
-
+         (accounts (append accounts
+                           (if dosubs?
+                               (filter (lambda (acc) (not (member acc accounts)))
+                                       (gnc:acccounts-get-all-subaccounts accounts))
+                               '())))
          (plot-type  (get-option gnc:pagename-display (N_ "Plot Type")))
          (show-plot? (get-option gnc:pagename-display (N_ "Show plot")))
          (show-table? (get-option gnc:pagename-display (N_ "Show table")))
@@ -326,30 +190,12 @@
 
 	 (commodity-list #f)
 	 (exchange-fn #f)
+         (all-zeros? #t))
 
-         (beforebegindate (gnc:timepair-end-day-time 
-                           (gnc:timepair-previous-day begindate)))
-         (all-zeros? #t)
-         ;; startbal will be a commodity-collector
-         (startbal  '()))
-
-    (define (list-all-zeros? alist)
-      (if (null? alist) #t
-          (if (not (= 0.0 (car alist)))
-              #f
-              (list-all-zeros? (cdr alist)))))
-    
-    (define (monetary->double foreign-monetary date)
-      (gnc-numeric-to-double
-       (gnc:gnc-monetary-amount
-        (exchange-fn foreign-monetary report-currency date))))
-
-    (gnc:html-document-set-title! document report-title)
     ;;(warn commodity-list)
 
     (if (not (null? accounts))
         (let ((query (qof-query-create-for-splits))
-              (splits '())
               (data '()))
 
           ;; The percentage done numbers here are a hack so that
@@ -360,11 +206,9 @@
           ;; lookup should be distributed and done when actually
           ;; needed so as to amortize the cpu time properly.
 	  (gnc:report-percent-done 1)
-	  (set! commodity-list (gnc:accounts-get-commodities 
-                                (append 
-                                 (gnc:acccounts-get-all-subaccounts accounts)
-                                 accounts)
-                                report-currency))
+	  (set! commodity-list (gnc:accounts-get-commodities
+                                accounts report-currency))
+
 	  (gnc:report-percent-done 5)
 	  (set! exchange-fn (gnc:case-exchange-time-fn 
                              price-source report-currency 
@@ -382,71 +226,157 @@
           ;; add accounts to the query (include subaccounts 
           ;; if requested)
 	  (gnc:report-percent-done 25)
-          (if dosubs? 
-              (let ((subaccts '()))
-                (for-each 
-                 (lambda (acct)
-                   (let ((this-acct-subs 
-                          (gnc-account-get-descendants-sorted acct)))
-                     (if (list? this-acct-subs)
-                         (set! subaccts 
-                               (append subaccts this-acct-subs)))))
-                 accounts)
-                ;; Beware: delete-duplicates is an O(n^2)
-                ;; algorithm. More efficient method: sort the list,
-                ;; then use a linear algorithm.
-                (set! accounts
-                      (delete-duplicates (append accounts subaccts)))))
-	  (gnc:report-percent-done 30)
 
           (xaccQueryAddAccountMatch query accounts QOF-GUID-MATCH-ANY QOF-QUERY-AND)
           
           ;; match splits between start and end dates 
-          (xaccQueryAddDateMatchTS
+          (xaccQueryAddDateMatchTT
            query #t begindate #t enddate QOF-QUERY-AND)
           (qof-query-set-sort-order query
 				    (list SPLIT-TRANS TRANS-DATE-POSTED)
 				    (list QUERY-DEFAULT-SORT)
 				    '())
           
-          ;; get the query results 
-          (set! splits (qof-query-run query))
 	  (gnc:report-percent-done 40)
-          
-          ;; find the net starting balance for the set of accounts 
-          (set! startbal 
-                (gnc:accounts-get-balance-helper 
-                 accounts 
-                 (lambda (acct) (gnc:account-get-comm-balance-at-date 
-                                 acct beforebegindate #f))
-                 (lambda (x) #f)))
-	  (gnc:report-percent-done 50)
 
-          (set! startbal 
-                (gnc-numeric-to-double
-                 (gnc:gnc-monetary-amount
-                  (gnc:sum-collector-commodity 
-                   startbal
-                   report-currency 
-                   (lambda (a b) 
-                     (exchange-fn a b beforebegindate))))))
-	  (gnc:report-percent-done 60)
-	  
-          ;; and analyze the data 
-          (set! data (analyze-splits splits startbal
-                                     begindate enddate 
-                                     stepsize monetary->double
-				     internal-included))
-	  (gnc:report-percent-done 70)
+          (let* ((splits (qof-query-run query))
+                 (daily-dates (gnc:make-date-list begindate enddate DayDelta))
+                 (interval-dates (gnc:make-date-list begindate enddate stepsize))
+                 (accounts-balances (map
+                                     (lambda (acc)
+                                       (gnc:account-get-balances-at-dates
+                                        acc daily-dates))
+                                     accounts))
+                 (balances (map
+                            (lambda (date accounts-balance)
+                              (gnc:gnc-monetary-amount
+                               (gnc:sum-collector-commodity
+                                (apply gnc:monetaries-add accounts-balance)
+                                report-currency
+                                (lambda (monetary target-curr)
+                                  (exchange-fn monetary target-curr date)))))
+                            daily-dates
+                            (apply zip accounts-balances))))
+            (qof-query-destroy query)
+
+            ;; this is a complicated tight loop. start with:
+            ;; daily-balances & daily-dates, interval-dates, and the
+            ;; splitlist. traverse the daily balances and splitlist
+            ;; until we cross an interval date boundary, then
+            ;; summarize the interval-balances and interval-amounts
+            (let loop ((results '())
+                       (interval-bals '())
+                       (interval-amts '())
+                       (splits splits)
+                       (daily-balances (cdr balances))
+                       (daily-dates (cdr daily-dates))
+                       (interval-start (car interval-dates))
+                       (interval-dates (cdr interval-dates)))
+              (cond
+
+               ;; daily-dates finished. job done. add details for
+               ;; last-interval which must be handled separately.
+               ((null? daily-dates)
+                (set! data
+                  (reverse!
+                   (cons (list
+                          (qof-print-date interval-start)
+                          (qof-print-date (car interval-dates))
+                          (/ (apply + interval-bals)
+                             (length interval-bals))
+                          (apply max interval-bals)
+                          (apply min interval-bals)
+                          (apply + (filter positive? interval-amts))
+                          (- (apply + (filter negative? interval-amts)))
+                          (apply + interval-amts))
+                         results))))
+
+               ;; first daily-date > first interval-date -- crossed
+               ;; interval boundary -- add interval details to results
+               ((> (car daily-dates) (car interval-dates))
+                (loop (cons (list
+                             (qof-print-date interval-start)
+                             (qof-print-date (decdate (car interval-dates)
+                                                      DayDelta))
+                             (/ (apply + interval-bals)
+                                (length interval-bals))
+                             (apply max interval-bals)
+                             (apply min interval-bals)
+                             (apply + (filter positive? interval-amts))
+                             (- (apply + (filter negative? interval-amts)))
+                             (apply + interval-amts))
+                            results)    ;process interval amts&bals
+                      '()               ;reset interval-bals
+                      '()               ;and interval-amts
+                      splits
+                      daily-balances
+                      daily-dates
+                      (car interval-dates)
+                      (cdr interval-dates)))
+
+               ;; we're still within interval, no more splits left
+               ;; within current interval. add daily balance to
+               ;; interval.
+               ((or (null? splits)
+                    (> (xaccTransGetDate (xaccSplitGetParent (car splits)))
+                       (car interval-dates)))
+                (loop results
+                      (cons (car daily-balances) interval-bals)
+                      interval-amts
+                      splits
+                      (cdr daily-balances)
+                      (cdr daily-dates)
+                      interval-start
+                      interval-dates))
+
+               ;; we're still within interval. 'internal' is
+               ;; disallowed; there are at least 2 splits remaining,
+               ;; both from the same transaction. skip them. NOTE we
+               ;; should really expand this conditional whereby all
+               ;; splits are internal, however the option is labelled
+               ;; as 2-splits only. best maintain current behaviour.
+               ((and (not internal-included)
+                     (pair? (cdr splits))
+                     (= 2 (xaccTransCountSplits (xaccSplitGetParent (car splits))))
+                     (equal? (xaccSplitGetParent (car splits))
+                             (xaccSplitGetParent (cadr splits))))
+                (loop results
+                      interval-bals
+                      interval-amts ;interval-amts unchanged
+                      (cddr splits) ;skip two splits.
+                      daily-balances
+                      daily-dates
+                      interval-start
+                      interval-dates))
+
+               ;; we're still within interval. there are splits
+               ;; remaining. add split details to interval-amts
+               (else
+                (loop results
+                      interval-bals
+                      (cons (gnc:gnc-monetary-amount
+                             (exchange-fn
+                              (gnc:make-gnc-monetary
+                               (xaccAccountGetCommodity
+                                (xaccSplitGetAccount (car splits)))
+                               (xaccSplitGetAmount (car splits)))
+                              report-currency
+                              (car interval-dates)))
+                            interval-amts) ;add split amt to list
+                      (cdr splits)         ;and loop to next split
+                      daily-balances
+                      daily-dates
+                      interval-start
+                      interval-dates)))))
+
+          (gnc:report-percent-done 70)
           
           ;; make a plot (optionally)... if both plot and table, 
           ;; plot comes first. 
           (if show-plot?
               (let ((barchart (gnc:make-html-barchart))
-                    (width (get-option gnc:pagename-display 
-                                       (N_ "Plot Width")))
-                    (height (get-option gnc:pagename-display 
-                                        (N_ "Plot Height")))
+                    (height (get-option gnc:pagename-display optname-plot-height))
+                    (width (get-option gnc:pagename-display optname-plot-width))
                     (col-labels '())
                     (col-colors '()))
                 (if (memq 'AvgBalPlot plot-type)
@@ -454,7 +384,7 @@
                         ((number-data
                           (map 
                            (lambda (row) (list-ref row 2)) data)))
-                      (if (not (list-all-zeros? number-data))
+                      (if (not (every zero? number-data))
                           (begin
                             (gnc:html-barchart-append-column! 
                              barchart
@@ -463,14 +393,14 @@
                                   (append col-labels 
                                           (list (list-ref columns 2))))
                             (set! col-colors
-                                  (append col-colors (list "blue")))
+                                  (append col-colors (list "#0074D9")))
                             (set! all-zeros? #f)))))
                 
                 
                 (if (memq 'GainPlot plot-type)
                     (let ((number-data 
                            (map (lambda (row) (list-ref row 7)) data)))
-                      (if (not (list-all-zeros? number-data))
+                      (if (not (every zero? number-data))
                           (begin
                             (gnc:html-barchart-append-column! 
                              barchart
@@ -479,7 +409,7 @@
                                   (append col-labels 
                                           (list (list-ref columns 7))))
                             (set! col-colors
-                                  (append col-colors (list "green")))
+                                  (append col-colors (list "#2ECC40")))
                             (set! all-zeros? #f)))))
 
                 (if (memq 'GLPlot plot-type)
@@ -489,8 +419,8 @@
                            (map (lambda (row) (list-ref row 6)) data)))
                       ;; debit column 
                       (if (not (and
-                                (list-all-zeros? debit-data)
-                                (list-all-zeros? credit-data)))
+                                (every zero? debit-data)
+                                (every zero? credit-data)))
                           (begin
                             (gnc:html-barchart-append-column! 
                              barchart
@@ -499,7 +429,7 @@
                                   (append col-labels 
                                           (list (list-ref columns 5))))
                             (set! col-colors
-                                  (append col-colors (list "black")))
+                                  (append col-colors (list "#111111")))
                             
                             ;; credit
                             (gnc:html-barchart-append-column! 
@@ -509,11 +439,12 @@
                                   (append col-labels 
                                           (list (list-ref columns 6))))
                             (set! col-colors
-                                  (append col-colors (list "red")))
+                                  (append col-colors (list "#FF4136")))
                             (set! all-zeros? #f)))))
                 
                 (if (not all-zeros?)
                     (begin
+                      (gnc:html-barchart-set-title! barchart report-title)
                       (gnc:html-barchart-set-col-labels! 
                        barchart col-labels)
                       (gnc:html-barchart-set-col-colors!
@@ -539,17 +470,15 @@
                  table columns)
                 (for-each
                  (lambda (row)
-                   (gnc:html-table-append-row! table row))
+                   (gnc:html-table-append-row!
+                    table
+                    (map
+                     gnc:make-html-table-cell/markup
+                     (list "date-cell" "date-cell"
+                           "number-cell" "number-cell" "number-cell"
+                           "number-cell" "number-cell" "number-cell")
+                     row)))
                  data)
-                
-                ;; set numeric columns to align right 
-                (for-each 
-                 (lambda (col)
-                   (gnc:html-table-set-col-style! 
-                    table col "td" 
-                    'attribute (list "align" "right")))
-                 '(2 3 4 5 6 7))
-                
                 (gnc:html-document-add-object! document table))))
 
         ;; if there are no accounts selected...
