@@ -42,8 +42,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define-module (gnucash report standard-reports income-statement))
-(use-modules (gnucash main)) ;; FIXME: delete after we finish modularizing.
-(use-modules (gnucash printf))
+(use-modules (gnucash utilities)) 
 (use-modules (gnucash gnc-module))
 (use-modules (gnucash gettext))
 
@@ -140,6 +139,7 @@
 ;; options generator
 (define (income-statement-options-generator-internal reportname)
   (let* ((options (gnc:new-options))
+         (book (gnc-get-current-book)) ; XXX Find a way to get the book that opened the report
          (add-option 
           (lambda (new-option)
             (gnc:register-option options new-option))))
@@ -151,7 +151,7 @@
     (add-option
       (gnc:make-string-option
       gnc:pagename-general optname-party-name
-      "b" opthelp-party-name (or (gnc:company-info gnc:*company-name*) "")))
+      "b" opthelp-party-name (or (gnc:company-info book gnc:*company-name*) "")))
     
     ;; period over which to report income
     (gnc:options-add-date-interval!
@@ -301,11 +301,11 @@
          (start-date-printable (gnc:date-option-absolute-time
 				(get-option gnc:pagename-general
 					    optname-start-date)))
-         (start-date-tp (gnc:timepair-start-day-time
+         (start-date (gnc:time64-start-day-time
 			 (gnc:date-option-absolute-time
 			  (get-option gnc:pagename-general
 				      optname-start-date))))
-         (end-date-tp (gnc:timepair-end-day-time
+         (end-date (gnc:time64-end-day-time
 		       (gnc:date-option-absolute-time
 			(get-option gnc:pagename-general
 				    optname-end-date))))
@@ -387,7 +387,7 @@
 			 depth-limit))
          ;; exchange rates calculation parameters
 	 (exchange-fn
-	  (gnc:case-exchange-fn price-source report-commodity end-date-tp))
+	  (gnc:case-exchange-fn price-source report-commodity end-date))
 	 )
     
     ;; Wrapper to call gnc:html-table-add-labeled-amount-line!
@@ -429,12 +429,12 @@
 	  (if (equal? tabbing 'canonically-tabbed) 1 0))))
     
     (gnc:html-document-set-title! 
-     doc (sprintf #f
-		  (string-append "%s %s "
-				 (_ "For Period Covering %s to %s"))
+     doc (format #f
+		  (string-append "~a ~a "
+				 (_ "For Period Covering ~a to ~a"))
 		  company-name report-title
-                  (gnc-print-date start-date-printable)
-                  (gnc-print-date end-date-tp)))
+                  (qof-print-date start-date-printable)
+                  (qof-print-date end-date)))
     
     (if (null? accounts)
 	
@@ -471,9 +471,9 @@
 	       (terse-period? #t)
 	       (period-for (if terse-period?
 			       (string-append " " (_ "for Period"))
-			       (sprintf #f (string-append ", " (_ "%s to %s"))
-					(gnc-print-date start-date-printable)
-					(gnc-print-date end-date-tp))
+			       (format #f (string-append ", " (_ "~a to ~a"))
+					(qof-print-date start-date-printable)
+					(qof-print-date end-date))
 			       )
 			   )
 	       )
@@ -517,29 +517,29 @@
 	  (set! revenue-closing
 		(gnc:account-get-trans-type-balance-interval-with-closing
 		 revenue-accounts closing-pattern
-		 start-date-tp end-date-tp)
+		 start-date end-date)
 		) ;; this is norm positive (debit)
 	  (set! expense-closing
 		(gnc:account-get-trans-type-balance-interval-with-closing
 		 expense-accounts closing-pattern
-		 start-date-tp end-date-tp)
+		 start-date end-date)
 		) ;; this is norm negative (credit)
 	  (set! expense-total
 		(gnc:accountlist-get-comm-balance-interval-with-closing
 		 expense-accounts
-		 start-date-tp end-date-tp))
+		 start-date end-date))
 	  (expense-total 'minusmerge expense-closing #f)
 	  (set! neg-revenue-total
 		(gnc:accountlist-get-comm-balance-interval-with-closing
 		 revenue-accounts
-		 start-date-tp end-date-tp))
+		 start-date end-date))
 	  (neg-revenue-total 'minusmerge revenue-closing #f)
 	  (set! revenue-total (gnc:make-commodity-collector))
 	  (revenue-total 'minusmerge neg-revenue-total #f)
           (set! trading-total 
                 (gnc:accountlist-get-comm-balance-interval-with-closing
                  trading-accounts
-                 start-date-tp end-date-tp))
+                 start-date end-date))
 	  ;; calculate net income
 	  (set! net-income (gnc:make-commodity-collector))
 	  (net-income 'merge revenue-total #f)
@@ -548,8 +548,8 @@
 	  
 	  (set! table-env
 		(list
-		 (list 'start-date start-date-tp)
-		 (list 'end-date end-date-tp)
+		 (list 'start-date start-date)
+		 (list 'end-date end-date)
 		 (list 'display-tree-depth tree-depth)
 		 (list 'depth-limit-behavior (if bottom-behavior
 						 'flatten
@@ -647,14 +647,16 @@
 		  (if standard-order?
 		      (list
 		       (gnc:make-html-table-cell inc-table)
-		       (if (not (null? trading-accounts))
+		       (if (null? trading-accounts)
+			   (gnc:html-make-empty-cell)
 		           (gnc:make-html-table-cell tra-table))
 		       (gnc:make-html-table-cell exp-table)
 		       )
 		      (list
 		       (gnc:make-html-table-cell exp-table)
 		       (gnc:make-html-table-cell inc-table)
-		       (if (not (null? trading-accounts))
+		       (if (null? trading-accounts)
+			   (gnc:html-make-empty-cell)
 		           (gnc:make-html-table-cell tra-table))
 		       )
 		      )
@@ -711,9 +713,7 @@
     
     (gnc:report-finished)
     
-    doc
-    )
-  )
+    doc))
 
 (define is-reportname (N_ "Income Statement"))
 (define pnl-reportname (N_ "Profit & Loss"))
@@ -735,8 +735,7 @@
  'report-guid "0b81a3bdfd504aff849ec2e8630524bc"
  'menu-path (list gnc:menuname-income-expense)
  'options-generator income-statement-options-generator
- 'renderer income-statement-renderer
- )
+ 'renderer income-statement-renderer)
 
 ;; Also make a "Profit & Loss" report, even if it's the exact same one,
 ;; just relabeled.
@@ -746,7 +745,6 @@
  'report-guid "8758ba23984c40dea5527f5f0ca2779e"
  'menu-path (list gnc:menuname-income-expense)
  'options-generator profit-and-loss-options-generator
- 'renderer profit-and-loss-renderer
- )
+ 'renderer profit-and-loss-renderer)
 
 ;; END
