@@ -53,7 +53,7 @@
   (if (or (not taxable) (eq? taxtable '()))
     (display "&nbsp;")
     (let* ((amttot  (gnc:make-commodity-collector))
-           (pctot   (gnc:make-number-collector)) 
+           (pctot   (gnc:make-value-collector))
            (entries (gncTaxTableGetEntries taxtable))
            (amt?    #f)  ; becomes #t if any entries are amounts
            (pc?     #f)) ; becomes #t if any entries are percentages
@@ -221,7 +221,7 @@
                 headingpage2 optname-amount-due "b" "" (_ "Amount Due")))
   (add-option (gnc:make-string-option
                 headingpage2 optname-payment-recd "c" "" 
-                (_ "Payment received, thank you.")))
+                (_ "Payment received, thank you!")))
   (add-option (gnc:make-string-option	headingpage2	optname-invoice-number-text
     "d" "" (_ "Invoice number: ")))
   (add-option (gnc:make-string-option	headingpage2	optname-to-text
