@@ -27,7 +27,9 @@
 ;; returns a list with n #f (empty cell) values 
 (define (gnc:html-make-empty-cell) #f)
 (define (gnc:html-make-empty-cells n)
-  (make-list n #f))
+  (if (> n 0)
+      (cons #f (gnc:html-make-empty-cells (- n 1)))
+      (list)))
 
 (define (gnc:register-guid type guid)
   (gnc-build-url URL-TYPE-REGISTER (string-append type guid) ""))
@@ -279,18 +281,15 @@
 	;; commodity
 	(commodity-row-helper! 
 	 my-name #f
-	 (if balance 
-	     (gnc-commodity-collector-assoc
-	      balance report-commodity reverse-balance?)
-	     #f)
+	 (and balance
+              (balance 'getmonetary report-commodity reverse-balance?))
 	 main-row-style)
 	;; Special case for stock-accounts: then the foreign commodity
 	;; gets displayed in this line rather then the following lines
 	;; (loop below). Is also used if is-stock-account? is true.
-	(let ((my-balance 
-	       (if balance 
-		   (gnc-commodity-collector-assoc
-		    balance my-commodity reverse-balance?) #f)))
+	(let ((my-balance
+               (and balance
+                    (balance 'getmonetary my-commodity reverse-balance?))))
 	  (set! already-printed my-commodity)
 	  (commodity-row-helper! 
 	   my-name
@@ -302,8 +301,8 @@
     ;; balance and its corresponding value in the
     ;; report-currency. One row for each non-report-currency. 
     (if (and balance (not is-stock-account?))
-	(gnc-commodity-collector-map
-	 balance 
+	(balance
+         'format
 	 (lambda (curr val)
 	   (if (or (gnc-commodity-equiv curr report-commodity)
 		   (and already-printed
@@ -321,7 +320,7 @@
 		  bal
 		  (exchange-fn bal report-commodity)
 		  other-rows-style))))
-	 ))))
+	 #f))))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -464,8 +463,7 @@
       (let ((this-collector (my-get-balance-nosub account)))
 	(for-each 
 	 (lambda (x) (if x 
-			 (gnc-commodity-collector-merge
-			  this-collector x )))
+			 (this-collector 'merge x #f)))
 	 (gnc:account-map-descendants
 	  (lambda (a)
 	    ;; Important: Calculate the balance if and only if the
@@ -637,7 +635,7 @@
 				  subaccounts my-get-balance 
 				  gnc-reverse-balance)))
 		 (if thisbalance 
-		     (gnc-commodity-collector-merge subbalance thisbalance))
+		     (subbalance 'merge thisbalance #f))
 		 subbalance)
 	       heading-style
 	       #t #f)))))
@@ -742,54 +740,23 @@
 ;; 'common-commodity', the exchange rates are given through the
 ;; function 'exchange-fn' and the 'accounts' determine which
 ;; commodities to show. Returns a html-object, a <html-table>.
-(define (gnc:html-make-exchangerates
-	 common-commodity exchange-fn accounts) 
-  (let ((comm-list 
-	 (gnc:accounts-get-commodities accounts common-commodity))
-	(table (gnc:make-html-table)))
-
-    (if (not (null? comm-list))
-	;; Do something with each exchange rate.
-	(begin
-	  (for-each 
-	   (lambda (commodity)
-	     (let 
-		 ;; slight hack: exchange a value greater than one,
-		 ;; to get enough digits, and round later.
-		 ((exchanged 
-		   (exchange-fn 
-		    (gnc:make-gnc-monetary commodity 
-					   (gnc-numeric-create 1000 1))
-		    common-commodity)))
-	       (gnc:html-table-append-row! 
-		table
-		(list 
-		 (gnc:make-gnc-monetary commodity 
-					(gnc-numeric-create 1 1))
-		 (gnc:make-gnc-monetary
-		  common-commodity
-		  (gnc-numeric-div
-		   (gnc:gnc-monetary-amount exchanged)
-		   (gnc-numeric-create 1000 1)
-		   GNC-DENOM-AUTO 
-		   (logior (GNC-DENOM-SIGFIGS 6) 
-			   GNC-RND-ROUND)))))))
-	   comm-list)
-	  
-	  ;; Set some style
-	  (gnc:html-table-set-style! 
-	   table "td" 
-	   'attribute '("align" "right")
-	   'attribute '("valign" "top"))
-	  
-	  ;; set some column headers 
-	  (gnc:html-table-set-col-headers!
-	   table 
-	   (list (gnc:make-html-table-header-cell/size 
-		  1 2 (if (= 1 (length comm-list))
-			  (_ "Exchange rate")
-			  (_ "Exchange rates")))))))
-    
+(define (gnc:html-make-exchangerates common-commodity exchange-fn accounts)
+  (let ((comm-list (gnc:accounts-get-commodities accounts common-commodity))
+        (markup (lambda (c) (gnc:make-html-table-cell/markup "number-cell" c)))
+        (table (gnc:make-html-table)))
+    (unless (null? comm-list)
+      (for-each
+       (lambda (commodity)
+         (let* ((orig-amt (gnc:make-gnc-monetary commodity 1))
+                (exchanged (exchange-fn orig-amt common-commodity)))
+           (gnc:html-table-append-row!
+            table (map markup (list orig-amt exchanged)))))
+       comm-list)
+      (gnc:html-table-set-col-headers!
+       table (list (gnc:make-html-table-header-cell/size
+                    1 2 (if (null? (cdr comm-list))
+                            (_ "Exchange rate")
+                            (_ "Exchange rates"))))))
     table))
 
 
@@ -814,8 +781,8 @@
     (gnc:html-markup-p
      (gnc:html-markup-anchor
       (gnc-build-url URL-TYPE-OPTIONS
-                     (format #f "report-id=~a" report-id)
-                     "")
+       (string-append "report-id=" (format #f "~a" report-id))
+       "")
       (_ "Edit report options")))))
 
 (define* (gnc:html-render-options-changed options #:optional plaintext?)
